<?php
/*-------------------------------------------------------
*
*   LiveStreet Engine Social Networking
*   Copyright © 2008 Mzhelskiy Maxim
*
*--------------------------------------------------------
*
*   Official site: www.livestreet.ru
*   Contact e-mail: rus.engine@gmail.com
*
*   GNU General Public License, version 2:
*   http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*
---------------------------------------------------------
*/

/**
 * Модуль потока событий на сайте
 *
 * @package modules.stream
 * @since 1.0
 */
class ModuleStream extends Module
{
    /** @var ModuleStream_MapperStream Объект маппера */
    protected ModuleStream_MapperStream $oMapper;

    /** @var int Идентификатор текущего пользователя */
    protected int $currentUserId = 0;
    /** @var bool Является ли текущий пользователь администратором */
    protected bool $currentUserIsAdmin = false;

    /** @var array Список дефолтных типов событий, они добавляются каждому пользователю при регистрации*/
    protected array $aEventDefaultTypes = ['add_wall','add_topic','add_comment','add_blog','vote_topic','add_friend'];

    /** @var array Типы событий */
    protected array $aEventTypes = [
        'add_topic' => ['related' => 'topic','unique'=>true],
        'add_comment' => ['related' => 'comment','unique'=>true],
        'add_blog' => ['related' => 'blog','unique'=>true],
        'vote_topic' => ['related' => 'topic'],
        'vote_comment' => ['related' => 'comment'],
        'vote_blog' => ['related' => 'blog'],
        'vote_user' => ['related' => 'user'],
        'add_friend' => ['related' => 'user','unique_user'=>true],
        'join_blog' => ['related' => 'blog','unique_user'=>true]
    ];

    /** @var array Описывает условия, которым должны соответствовать объекты отображаемых событий. */
    protected array $eventTargetPublishingTerms = [
        'topic'=>['topic_publish'=>true],
        'comment'=>['target_type'=>'"topic"','comment_delete'=>0,'comment_publish'=>true],
        //'blog'=>[],
    ];
    /** @var array Указывает, как связывать сущность с блогом (для проверки её доступности на основании закрытости блога). */
    protected array $eventTargetToBlogRelation = [];

    /**
     * Инициализация модуля
     */
    public function Init()
    {
        // указывает, как связывать сущность с блогом (для проверки её доступности на основании закрытости блога).
        // массив, в котором ключ — имя таблицы, значение — имена индексного поля и поля в этой таблице, по которому связывать с индексным полем следущей по списку таблицы
        // и так далее до таблицы блога с его идентификатором (это уже указывать не надо). массив отсутствует (не пустой, а именно отсутствует) — связности с блогом нет.
        $this->eventTargetToBlogRelation = [
            'comment' => [Config::Get('db.table.comment')=>['comment_id','target_parent_id']],
            'topic' => [Config::Get('db.table.topic')=>['topic_id','blog_id']],
            'blog' => [],
        ];

        $this->oMapper=Engine::GetMapper(__CLASS__);

        if($this->User_getUserCurrent()) {
            $this->currentUserId = $this->User_getUserCurrent()->getId();
            $this->currentUserIsAdmin = $this->User_getUserCurrent()->isAdministrator();
        }
    }
    /**
     * Возвращает все типы событий
     *
     * @return array
     */
    public function getEventTypes()
    {
        return $this->aEventTypes;
    }
    /**
     * Возвращает типы событий с учетом фильтра(доступности)
     *
     * @param array|null $aTypes	Список типов
     * @return array
     */
    public function getEventTypesFilter($aTypes=null)
    {
        if (is_null($aTypes)) {
            $aTypes=array_keys($this->getEventTypes());
        }
        if (Config::Get('module.stream.disable_vote_events')) {
            foreach ($aTypes as $i => $sType) {
                if (substr($sType, 0, 4) == 'vote') {
                    unset($aTypes[$i]);
                }
            }
        }
        return $aTypes;
    }
    /**
     * Добавляет новый тип события, метод для расширения списка событий плагинами
     *
     * @param string $sName	Название типа
     * @param array $aParams	Параметры
     * @return bool
     */
    public function AddEventType($sName, $aParams)
    {
        if (!array_key_exists($sName, $this->aEventTypes)) {
            $this->aEventTypes[$sName]=$aParams;
            return true;
        }
        return false;
    }
    /**
     * Проверка допустимого типа событий
     *
     * @param string $sType	Тип
     * @return bool
     */
    public function IsAllowEventType($sType)
    {
        if (!is_string($sType)) {
            return false;
        }
        return array_key_exists($sType, $this->aEventTypes);
    }

    /**
     * @return array
     */
    public function getEventTargetPublishingTerms(): array
    {
        return $this->eventTargetPublishingTerms;
    }
    /**
     * @return array
     */
    public function getEventTargetToBlogRelation(): array
    {
        return $this->eventTargetToBlogRelation;
    }

    /**
     * Добавление события в БД
     *
     * @param ModuleStream_EntityEvent $oObject	Объект события
     * @return ModuleStream_EntityEvent|bool
     */
    public function AddEvent($oObject)
    {
        if ($iId=$this->oMapper->AddEvent($oObject)) {
            $oObject->setId($iId);
            return $oObject;
        }
        return false;
    }
    /**
     * Обновление события
     *
     * @param ModuleStream_EntityEvent $oObject	Объект события
     * @return int
     */
    public function UpdateEvent($oObject)
    {
        return $this->oMapper->UpdateEvent($oObject);
    }
    /**
     * Получает событие по типу и его ID
     *
     * @param string $sEventType	Тип
     * @param int $iTargetId	ID владельца события
     * @param int|null $iUserId	ID пользователя
     * @return ModuleStream_EntityEvent
     */
    public function GetEventByTarget($sEventType, $iTargetId, $iUserId=null)
    {
        return $this->oMapper->GetEventByTarget($sEventType, $iTargetId, $iUserId);
    }
    /**
     * Запись события в ленту
     *
     * @param int $iUserId	ID пользователя
     * @param string $sEventType	Тип события
     * @param int $iTargetId	ID владельца
     * @param int $iPublish	Статус
     * @return bool
     */
    public function Write($iUserId, $sEventType, $iTargetId, $iPublish=1)
    {
        $iPublish=(int)$iPublish;
        if (!$this->IsAllowEventType($sEventType)) {
            return false;
        }
        $aParams=$this->aEventTypes[$sEventType];
        if (isset($aParams['unique']) and $aParams['unique']) {
            /**
             * Проверяем на уникальность
             */
            if ($oEvent=$this->GetEventByTarget($sEventType, $iTargetId)) {
                /**
                 * Событие уже было
                 */
                if ($oEvent->getPublish()!=$iPublish) {
                    $oEvent->setPublish($iPublish);
                    $this->UpdateEvent($oEvent);
                }
                return true;
            }
        }
        if (isset($aParams['unique_user']) and $aParams['unique_user']) {
            /**
             * Проверяем на уникальность для конкретного пользователя
             */
            if ($oEvent=$this->GetEventByTarget($sEventType, $iTargetId, $iUserId)) {
                /**
                 * Событие уже было
                 */
                if ($oEvent->getPublish()!=$iPublish) {
                    $oEvent->setPublish($iPublish);
                    $this->UpdateEvent($oEvent);
                }
                return true;
            }
        }

        if ($iPublish) {
            /**
             * Создаем новое событие
             */
            $oEvent=Engine::GetEntity('Stream_Event');
            $oEvent->setEventType($sEventType);
            $oEvent->setUserId($iUserId);
            $oEvent->setTargetId($iTargetId);
            $oEvent->setDateAdded(date("Y-m-d H:i:s"));
            $oEvent->setPublish($iPublish);
            $this->AddEvent($oEvent);
        }
        return true;
    }
    /**
     * Чтение потока пользователя
     *
     * @param int|null $iCount	Количество
     * @param int|null $iFromId	ID события с которого начинать выборку
     * @param int|null $iUserId	ID пользователя
     * @param bool $remainingEvents Переменная, переданная сюда по ссылке, будет установлена в true, если по заданным критериям остались события за рамками $iCount.
     * @return array
     */
    public function Read($iCount=null, $iFromId=null, $iUserId=null, bool &$remainingEvents = false)
    {
        if (!$iUserId) {
            if ($this->User_getUserCurrent()) {
                $iUserId=$this->User_getUserCurrent()->getId();
            } else {
                return array();
            }
        }
        // Получаем типы событий
        $aEventTypes = $this->getTypesList($iUserId);
        // Получаем список тех на кого подписан
        $aUsersList = $this->getUsersList($iUserId);

        return $this->ReadEvents($aEventTypes, $aUsersList, $iCount, $iFromId, $remainingEvents);
    }
    /**
     * Чтение всей активности на сайте
     *
     * @param int|null $iCount	Количество
     * @param int|null $iFromId	ID события с которого начинать выборку
     * @param bool $remainingEvents Переменная, переданная сюда по ссылке, будет установлена в true, если по заданным критериям остались события за рамками $iCount.
     * @return array
     */
    public function ReadAll($iCount=null, $iFromId=null, bool &$remainingEvents = false)
    {
        // Получаем типы событий
        $aEventTypes=array_keys($this->getEventTypes());

        return $this->ReadEvents($aEventTypes, null, $iCount, $iFromId, $remainingEvents);
    }
    /**
     * Чтение активности конкретного пользователя
     *
     * @param int $iUserId	ID пользователя
     * @param int|null $iCount	Количество
     * @param int|null $iFromId	ID события с которого начинать выборку
     * @param bool $remainingEvents Переменная, переданная сюда по ссылке, будет установлена в true, если по заданным критериям остались события за рамками $iCount.
     * @return array
     */
    public function ReadByUserId($iUserId, $iCount=null, $iFromId=null, bool &$remainingEvents = false)
    {
        // Получаем типы событий
        $aEventTypes=array_keys($this->getEventTypes());
        $aUsersList = [$iUserId];

        return $this->ReadEvents($aEventTypes, $aUsersList, $iCount, $iFromId, $remainingEvents);
    }
    /**
     * Количество событий конкретного пользователя
     * @deprecated Используйте механизм бесконечной ленты для загрузки событий. Всегда возвращает 0.
     * @param int $iUserId	ID пользователя
     * @return int
     */
    public function GetCountByUserId($iUserId)
    {
        return 0;
        /**
         * Получаем типы событий
         */
        $aEventTypes=$this->getEventTypesFilter();
        if (!count($aEventTypes)) {
            return 0;
        }

        return $this->oMapper->GetCount($aEventTypes, $iUserId, $this->currentUserId, $this->currentUserIsAdmin);
    }
    /**
     * Количество событий на которые подписан пользователь
     * @deprecated Используйте механизм бесконечной ленты для загрузки событий. Всегда возвращает 0.
     * @param int $iUserId	ID пользователя
     * @return int
     */
    public function GetCountByReaderId($iUserId)
    {
        return 0;
        /**
         * Получаем типы событий
         */
        $aEventTypes=$this->getEventTypesFilter($this->getTypesList($iUserId));
        if (!count($aEventTypes)) {
            return 0;
        }
        /**
         * Получаем список тех на кого подписан
         */
        $aUsersList = $this->getUsersList($iUserId);

        return $this->oMapper->GetCount($aEventTypes, $aUsersList, $this->currentUserId, $this->currentUserIsAdmin);
    }
    /**
     * Количество событий на всем сайте
     * @deprecated Используйте механизм бесконечной ленты для загрузки событий. Всегда возвращает 0.
     * @return int
     */
    public function GetCountAll()
    {
        return 0;
        /**
         * Получаем типы событий
         */
        $aEventTypes=$this->getEventTypesFilter();
        if (!count($aEventTypes)) {
            return 0;
        }

        return $this->oMapper->GetCount($aEventTypes, null, $this->currentUserId, $this->currentUserIsAdmin);
    }
    /**
     * Количество событий для пользователя
     * @deprecated Используйте механизм бесконечной ленты для загрузки событий. Всегда возвращает 0.
     * @param array $aEventTypes	Список типов событий
     * @param array|null $aUserId	ID пользователя
     * @return int
     */
    public function GetCount($aEventTypes, $aUserId=null)
    {
        return 0;
        return $this->oMapper->GetCount($aEventTypes, $aUserId, $this->currentUserId, $this->currentUserIsAdmin);
    }
    /**
     * Чтение событий
     *
     * @param array $aEventTypes	Список типов событий
     * @param array|null $aUsersList	Список пользователей, чьи события читать
     * @param int $iCount	Количество
     * @param int $iFromId	ID события с которого начинать выборку
     * @param bool $remainingEvents Переменная, переданная сюда по ссылке, будет установлена в true, если по заданным критериям остались события за рамками $iCount.
     * @return array
     */
    public function ReadEvents($aEventTypes, $aUsersList, $iCount=null, $iFromId=null, bool &$remainingEvents = false)
    {
        if (!is_null($aUsersList) and !count($aUsersList)) {
            return [];
        }
        if (!$iCount) {
            $iCount = Config::Get('module.stream.count_default');
        }

        $aEventTypes=$this->getEventTypesFilter($aEventTypes);
        if (!count($aEventTypes)) {
            return [];
        }
        // Получаем список событий. Количество событий указываем +1, чтобы узнать, есть ли записи после лимита
        $aEvents = $this->oMapper->Read($aEventTypes, $aUsersList, $iCount+1, $iFromId, $this->currentUserId, $this->currentUserIsAdmin);
        $remainingEvents = false;
        if(count($aEvents) > $iCount) {
            $remainingEvents = true;
            unset($aEvents[count($aEvents)-1]); // удаляем лишний результат из возвращаемой выборки
        }
        // Составляем список объектов для загрузки
        $aNeedObjects = [];
        foreach ($aEvents as $oEvent) {
            if (isset($this->aEventTypes[$oEvent->getEventType()]['related'])) {
                $aNeedObjects[$this->aEventTypes[$oEvent->getEventType()]['related']][]=$oEvent->getTargetId();
            }
            $aNeedObjects['user'][]=$oEvent->getUserId();
        }
        // Получаем объекты
        $aObjects = [];
        foreach ($aNeedObjects as $sType => $aListId) {
            if (count($aListId)) {
                $aListId=array_unique($aListId);
                $sMethod = 'loadRelated' . ucfirst($sType);
                if (method_exists($this, $sMethod)) {
                    if ($aRes=$this->$sMethod($aListId)) {
                        foreach ($aRes as $oObject) {
                            $aObjects[$sType][$oObject->getId()]=$oObject;
                        }
                    }
                }
            }
        }
        // Формируем результирующий поток
        foreach ($aEvents as $key => $oEvent) {
            // Жестко вытаскиваем автора события
            if (isset($aObjects['user'][$oEvent->getUserId()])) {
                $oEvent->setUser($aObjects['user'][$oEvent->getUserId()]);
                $aVote=array();
                if ($oEvent->getEventType() === 'vote_topic') {
                    $aVote=$this->Vote_GetVoteByArray($oEvent->getTargetId(), 'topic', $oEvent->getUserId());
                    $oEvent->setVote($aVote[$oEvent->getTargetId()]);
                } elseif ($oEvent->getEventType() === 'vote_comment') {
                    $aVote=$this->Vote_GetVoteByArray($oEvent->getTargetId(), 'comment', $oEvent->getUserId());
                    $oEvent->setVote($aVote[$oEvent->getTargetId()]);
                } elseif ($oEvent->getEventType() === 'vote_user') {
                    $aVote=$this->Vote_GetVoteByArray($oEvent->getTargetId(), 'user', $oEvent->getUserId());
                    $oEvent->setVote($aVote[$oEvent->getTargetId()]);
                } elseif ($oEvent->getEventType() === 'vote_blog') {
                    $aVote=$this->Vote_GetVoteByArray($oEvent->getTargetId(), 'blog', $oEvent->getUserId());
                    $oEvent->setVote($aVote[$oEvent->getTargetId()]);
                }
                // Аттачим объекты
                if (isset($this->aEventTypes[$oEvent->getEventType()]['related'])) {
                    $sTypeObject=$this->aEventTypes[$oEvent->getEventType()]['related'];
                    if (isset($aObjects[$sTypeObject][$oEvent->getTargetId()])) {
                        $oEvent->setTarget($aObjects[$sTypeObject][$oEvent->getTargetId()]);
                    } else {
                        unset($aEvents[$key]);
                    }
                } else {
                    unset($aEvents[$key]);
                }
            } else {
                unset($aEvents[$key]);
            }
        }
        return $aEvents;
    }
    /**
     * Получение типов событий, на которые подписан пользователь
     *
     * @param int $iUserId	ID пользователя
     * @return array
     */
    public function getTypesList($iUserId)
    {
        return $this->oMapper->getTypesList($iUserId);
    }
    /**
     * Получение списка id пользователей, на которых подписан пользователь
     *
     * @param int $iUserId	ID пользователя
     * @return array
     */
    protected function getUsersList($iUserId)
    {
        return $this->oMapper->getUserSubscribes($iUserId);
    }
    /**
     * Получение списка пользователей, на которых подписан пользователь
     *
     * @param int $iUserId	ID пользователя
     * @return array
     */
    public function getUserSubscribes($iUserId)
    {
        $aIds = $this->oMapper->getUserSubscribes($iUserId);
        return $this->User_GetUsersAdditionalData($aIds);
    }
    /**
     * Проверяет подписан ли пользователь на конкретного пользователя
     *
     * @param int $iUserId	ID пользователя
     * @param int $iTargetUserId	ID пользователя на которого подписан
     * @return bool
     */
    public function IsSubscribe($iUserId, $iTargetUserId)
    {
        return $this->oMapper->IsSubscribe($iUserId, $iTargetUserId);
    }
    /**
     * Редактирование списка событий, на которые подписан юзер
     *
     * @param int $iUserId	ID пользователя
     * @param string $sType	Тип
     * @return bool
     */
    public function switchUserEventType($iUserId, $sType)
    {
        if ($this->IsAllowEventType($sType)) {
            return $this->oMapper->switchUserEventType($iUserId, $sType);
        }
        return false;
    }
    /**
     * Переключает дефолтный список типов событий у пользователя
     *
     * @param int $iUserId	ID пользователя
     */
    public function switchUserEventDefaultTypes($iUserId)
    {
        foreach ($this->aEventDefaultTypes as $sType) {
            $this->switchUserEventType($iUserId, $sType);
        }
    }
    /**
     * Подписать пользователя
     *
     * @param int $iUserId	ID пользователя
     * @param int $iTargetUserId	ID пользователя на которого подписываем
     */
    public function subscribeUser($iUserId, $iTargetUserId)
    {
        $this->oMapper->subscribeUser($iUserId, $iTargetUserId);
    }
    /**
     * Отписать пользователя
     *
     * @param int $iUserId	ID пользователя
     * @param int $iTargetUserId	ID пользователя на которого подписываем
     */
    public function unsubscribeUser($iUserId, $iTargetUserId)
    {
        $this->oMapper->unsubscribeUser($iUserId, $iTargetUserId);
    }
    /**
     * Получает список топиков
     *
     * @param array $aIds	Список  ID топиков
     * @return array
     */
    protected function loadRelatedTopic($aIds)
    {
        return $this->Topic_GetTopicsAdditionalData($aIds);
    }
    /**
     * Получает список блогов
     *
     * @param array $aIds	Список  ID блогов
     * @return array
     */
    protected function loadRelatedBlog($aIds)
    {
        return $this->Blog_GetBlogsAdditionalData($aIds);
    }
    /**
     * Получает список комментариев
     *
     * @param array $aIds	Список  ID комментариев
     * @return array
     */
    protected function loadRelatedComment($aIds)
    {
        return $this->Comment_GetCommentsAdditionalData($aIds);
    }
    /**
     * Получает список пользователей
     *
     * @param array $aIds	Список  ID пользователей
     * @return array
     */
    protected function loadRelatedUser($aIds)
    {
        return $this->User_GetUsersAdditionalData($aIds);
    }
}
